﻿Imports System.Linq
Imports System.Text
Imports System.Collections
Imports System.Reflection

Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core

Namespace SampleSupport
  Public Class TreeViewGenerator

	Private Sub New()

	End Sub

	Public Shared ReadOnly Property Instance() As TreeViewGenerator
	  Get
		  Return __instance
	  End Get
	End Property

	Public Property ExpandDepth() As Integer
	Public Property MaxLoadDepth() As Integer

	Public Function GetTreeNode(ByVal o As Object) As TreeNode

	  If o Is Nothing Then
		Return GetNullNode()
	  End If

	  If TypeOf o Is Exception Then
		Return GetExceptionNode(CType(o, Exception))
	  End If

	  If TypeOf o Is Date Then
		Return GetDateTimeNode(CDate(o))
	  End If

	  If TypeOf o Is String Then
		Return GetStringNode(CStr(o))
	  End If

	  If TypeOf o Is Type Then
		Return GetTypeNode(CType(o, Type))
	  End If

	  If TypeOf o Is Byte() Then
		Return GetByteArrayNode(CType(o, Byte()))
	  End If

	  If TypeOf o Is ValueType Then
		Return GetValueTypeNode(o)
	  End If

	  If TypeOf o Is PropertyRef Then
		Return GetPropertyRefNode(CType(o, PropertyRef))
	  End If

	  If TypeOf o Is IEnumerable Then
		Return GetEnumerableNode(o)
	  End If

	  Return GetNestedObjectNode(o)
	End Function

	Public Sub ExpandTreeNode(ByVal parentNode As TreeNode)
	  If parentNode.Nodes.Count > 1 Then
		  Return
	  End If

	  Dim o = parentNode.Tag
	  If o Is Nothing Then
		  Return
	  End If
	  parentNode.Tag = Nothing ' insure we don't eval again;
	  parentNode.Nodes.Clear()
	  Dim members As IEnumerable(Of Member)
	  If TypeOf o Is PropertyRef Then
		Dim propRef = TryCast(o, PropertyRef)
		Dim eval = propRef.Evaluate()
		If TypeOf eval Is IEnumerable Then
		  parentNode.ImageIndex = 4
		  parentNode.SelectedImageIndex = parentNode.ImageIndex
		  Dim itermembers = (
		      From element As Object In (TryCast(eval, IEnumerable))
		      Select New Member With {.Name = Nothing, .Value = element}).ToList()

		  AttachChildren(parentNode, itermembers)

		  Return
		Else
		  members = BuildMembers(propRef.Evaluate())
		End If
	  Else
		members = BuildMembers(o)
	  End If
	  AttachChildren(parentNode, members)


	End Sub

	Private Function GetNestedObjectNode(ByVal o As Object) As TreeNode
	  Dim parentNode = BuildDefaultParentNode(o)
	  parentNode.ImageIndex = 1
	  parentNode.SelectedImageIndex = parentNode.ImageIndex
	  parentNode.Tag = o
	  parentNode.Nodes.Add("dummyNode") ' needed to show '+' sign
	  ' currently only updates text for entities - don't know what else can get in here
	  UpdateNodeText(o, parentNode)
	  Return parentNode
	End Function

	Private Function GetPropertyRefNode(ByVal pref As PropertyRef) As TreeNode
      Dim parentNode As New TreeNode(GetNiceTypeName(pref.Property1.PropertyType) & " (Lazy)")
	  parentNode.Tag = pref
	  parentNode.Nodes.Add("dummyNode") ' needed to show '+' sign
	  Return parentNode
	End Function

	Private Function GetEnumerableNode(ByVal o As Object) As TreeNode
	  Dim parentNode = BuildDefaultParentNode(o)
	  parentNode.ImageIndex = 4
	  parentNode.SelectedImageIndex = parentNode.ImageIndex
	  Dim members = (
	      From element As Object In (TryCast(o, IEnumerable))
	      Select New Member With {.Name = Nothing, .Value = element}).ToList()

	  AttachChildren(parentNode, members)

	  parentNode.Text &= " (" & members.Count & " item" & (If(members.Count <> 1, "s", "")) & ")"
	  Return parentNode
	End Function

	Private Function BuildDefaultParentNode(ByVal o As Object) As TreeNode
	  Dim parentNode As New TreeNode(GetNiceTypeName(o.GetType()))

	  If TypeOf o Is ObjectCollectionCache Then
		parentNode.Text = GetNiceTypeName((CType(o, ObjectCollectionCache)).OriginalType)
	  End If
	  Return parentNode
	End Function

	Private Shared Sub UpdateNodeText(ByVal o As Object, ByVal parentNode As TreeNode)
	  Dim ewk As Entity = TryCast(o, Entity)
	  If ewk IsNot Nothing Then
		Dim sb As New StringBuilder(parentNode.Text.Length + 20)
		sb.Append(parentNode.Text)
		sb.Append("(")

		Dim isFirst As Boolean = True
		For Each prop In ewk.EntityAspect.EntityMetadata.KeyProperties
		  If isFirst Then
			isFirst = False
		  Else
			sb.Append(", ")
		  End If
		  Dim pkColumnName As String = prop.Name
		  sb.Append(pkColumnName)
		  sb.Append("=")
		  sb.Append(ewk.EntityAspect.GetValue(pkColumnName).ToString())
		Next prop
		sb.Append(")")
		parentNode.Text = sb.ToString()
	  End If
	End Sub

	Private Function GetNiceTypeName(ByVal t As Type) As String
	  If t.Name.Contains("AnonymousType") Then
		Return "(AnonymousType)"
	  End If

	  If t.Name.Contains("SelectIterator") Then
		Return "(SelectIterator)"
	  End If

	  If t.IsGenericType Then
		Dim name As String = t.Name
		Dim backquote As Integer = name.IndexOf("`"c)
		If backquote >= 0 Then
		  name = name.Substring(0, backquote)
		End If

		Dim sb As New StringBuilder()
		sb.Append(name)
		sb.Append("<")
		Dim args() As Type = t.GetGenericArguments()
		For i As Integer = 0 To args.Length - 1
		  If i > 0 Then
			  sb.Append(", ")
		  End If
		  sb.Append(GetNiceTypeName(args(i)))
		Next i
		sb.Append(">")
		Return sb.ToString()
	  End If

	  Return t.Name
	End Function



	Public Shared Function BuildMembers(ByVal o As Object) As IEnumerable(Of Member)

      Dim memberList = New List(Of Member)

	  For Each member In o.GetType().GetMembers(BindingFlags.Public Or BindingFlags.Instance)
		Dim [property] = TryCast(member, PropertyInfo)
		If [property] Is Nothing Then
			Continue For
		End If
		If [property].DeclaringType Is GetType(Entity) Then
		  If [property].Name <> "IsNullEntity" AndAlso [property].Name <> "EntityMetadata" Then
			  Continue For
		  End If
		End If
		If [property].DeclaringType Is GetType(ComplexObject) Then
			Continue For
		End If
		If [property].Name.Equals("Item") Then ' indexer
			Continue For
		End If

		If GetType(Entity).IsAssignableFrom([property].PropertyType) AndAlso [property].Name.Equals("ScalarValue") Then
'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
          'yield Return New Member With {.Name = [property].Name, .Value = New PropertyRef(o, TryCast(member, PropertyInfo))}
          memberList.Add(New Member With {.Name = [property].Name, .Value = New PropertyRef(o, TryCast(member, PropertyInfo))})
          Continue For
		End If

		If GetType(Entity).IsAssignableFrom([property].PropertyType) AndAlso [property].Name.Equals("ListValue") Then
'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
          'yield Return New Member With {.Name = [property].Name, .Value = New PropertyRef(o, TryCast(member, PropertyInfo))}
          memberList.Add(New Member With {.Name = [property].Name, .Value = New PropertyRef(o, TryCast(member, PropertyInfo))})
          Continue For
		End If

		' if property returns a RelatedEntityList<>
		If GetType(IEnumerable).IsAssignableFrom([property].PropertyType) AndAlso [property].PropertyType.IsGenericType Then
		  Dim elementType = TypeFns.GetGenericArgument([property].PropertyType)
		  Dim genPropertyType = [property].PropertyType.GetGenericTypeDefinition()
		  If genPropertyType.Equals(GetType(RelatedEntityList(Of ))) Then
'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
            'yield Return New Member With {.Name = [property].Name, .Value = New PropertyRef(o, TryCast(member, PropertyInfo))}
            memberList.Add(New Member With {.Name = [property].Name, .Value = New PropertyRef(o, TryCast(member, PropertyInfo))})
            Continue For
		  End If
		End If

		Dim val = [property].GetValue(o, Nothing)
		If val IsNot Nothing Then
		  ' if (typeof(Entity).IsAssignableFrom(val.GetType())) continue;
		  If GetType(EntityGroup).IsAssignableFrom(val.GetType()) Then
			  Continue For
		  End If
		  If GetType(EntityQueryProxy).IsAssignableFrom(val.GetType()) Then
			  Continue For
		  End If
		  If GetType(EntityManager).IsAssignableFrom(val.GetType()) Then
			  Continue For
		  End If
		  If GetType(DataColumn).IsAssignableFrom(val.GetType()) Then
			  Continue For
		  End If
		  If GetType(EntityProperty).IsAssignableFrom(val.GetType()) Then
			  Continue For
		  End If
		End If

		Dim m = New Member With {.Name = [property].Name, .Value = val}
'INSTANT VB TODO TASK: VB does not support iterators and has no equivalent to the C# 'yield' keyword:
        'yield Return m
        memberList.Add(m)
      Next member
      Return memberList
	End Function

	Private Sub AttachChildren(ByVal parentNode As TreeNode, ByVal members As IEnumerable(Of Member))
	  Dim counter As Integer = 0

	  For Each m As Member In members
		Dim valueNode As TreeNode
		If parentNode.Level >= MaxLoadDepth Then
		  valueNode = New TreeNode("{ ... }")
		Else
		  valueNode = GetTreeNode(m.Value)
		  If m.Name IsNot Nothing Then
			valueNode.Text = m.Name & " = " & valueNode.Text
		  Else
			valueNode.Text = "#" & counter & ": " & valueNode.Text
			counter += 1
		  End If
		End If
		parentNode.Nodes.Add(valueNode)
	  Next m

	  If parentNode.Level < ExpandDepth Then
		parentNode.Expand()
	  End If
	End Sub

	Private Function GetDateTimeNode(ByVal dateTime As Date) As TreeNode
	  Return New TreeNode(dateTime.ToShortDateString())
	End Function

	Private Function GetNullNode() As TreeNode
	  Return New TreeNode("null")
	End Function

	Private Function GetStringNode(ByVal s As String) As TreeNode
	  Return New TreeNode("""" & s & """")
	End Function

	Private Function GetValueTypeNode(ByVal o As Object) As TreeNode
	  Return New TreeNode(Convert.ToString(o))
	End Function

	Private Function GetExceptionNode(ByVal ex As Exception) As TreeNode
	  Dim node As New TreeNode(ex.GetType().Name & ": " & ex.Message)
	  node.Nodes.Add(New TreeNode("Source: " & ex.Source))
	  Return node
	End Function

	Private Function GetTypeNode(ByVal t As Type) As TreeNode
	  Dim node As New TreeNode(GetNiceTypeName(t))
	  Return node
	End Function

	Private Function GetByteArrayNode(ByVal arr() As Byte) As TreeNode
	  Dim length As Integer = Math.Min(arr.Length, 32)
	  Dim t As String = "Byte[" & arr.Length & "] = " & BitConverter.ToString(arr, 0, length) + (If(length <> arr.Length, "...", ""))
	  Dim node As New TreeNode(t)
	  Return node
	End Function

	Private Shared __instance As New TreeViewGenerator()

	Public Class Member
	  Public Name As String
	  Public Value As Object
	End Class

	Friend Class PropertyRef
	  Public Sub New(ByVal obj As Object, ByVal [property] As PropertyInfo)
		BaseObject = obj
        Me.Property1 = [property]
	  End Sub
	  Private privateBaseObject As Object
	  Public Property BaseObject() As Object
		  Get
			  Return privateBaseObject
		  End Get
		  Private Set(ByVal value As Object)
			  privateBaseObject = value
		  End Set
	  End Property
	  Private privateProperty As PropertyInfo
      Private privateProperty1 As PropertyInfo
      Public Property Property1() As PropertyInfo
        Get
          Return privateProperty1
        End Get
        Private Set(ByVal value As PropertyInfo)
          privateProperty1 = value
        End Set
      End Property

	  Public Function Evaluate() As Object
        Return [Property1].GetValue(BaseObject, Nothing)
	  End Function
	End Class
  End Class
End Namespace
